#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Script to create and update a repository on zope instances
# Stéphan Geulette <stephan.geulette@uvcw.be>, UVCW
# From original script "recover.py" ()
#

import os, sys
import urllib
from datetime import datetime, timedelta
import socket
from utils import *

buildout_inst_type = None #True for buildout, False for manual instance
tempdir = ''
now = datetime(1973,02,12).now()
pfolders = {}
temp_added = False


###############################################################################

def main():
    global instdir, tempdir, buildout_inst_type, user, pwd

    instdir = instdir.rstrip('/')
    verbose("Working on instance %s" % instdir)    #must be commented when CPUtils updated in all instances

    # Finding the instance type (buildout or manual)
    if os.path.exists(os.path.join(instdir,'parts')):
        buildout_inst_type = True 
        #verbose("\tInstance is a buildout !")
    elif os.path.exists(os.path.join(instdir,'etc')):
        buildout_inst_type = False
        verbose("%s, manual installation !"%instdir)
    elif not os.path.exists(instdir) or True:
        error("%s, Invalid instance path or instance type not detected"%instdir)
        sys.exit(1)

    if buildout_inst_type:
        inst_type = 'buildout'
        zodbfilename = os.path.join(instdir, 'parts/instance/etc/zope.conf')
        fspath = os.path.join(instdir, 'var/filestorage/')
    else:
        inst_type = 'manual'
        zodbfilename = os.path.join(instdir, 'etc/zope.conf')
        fspath = os.path.join(instdir, 'var/')

    instance = os.path.basename(instdir)
    if not tempdir:
        tempdir = os.path.join(instdir, 'temp')
    hostname = socket.gethostname()

    port = treat_zopeconflines(zodbfilename, fspath)

    trace("host='%s', inst='%s'"%(hostname, instance))
    
    param='?instdir=%s'%instdir
    if instance.find('test') < 0:
        param+='&isProductInstance=1'
    CreateAndCallExternalMethod(port,user,pwd,'cputils_checkInstance','utils.py','checkInstance',param)

#------------------------------------------------------------------------------
try:
    from optparse import OptionParser
    parser = OptionParser()
    parser.add_option("-i", "--infos", dest="infos",
                  default=None,
                  help="infos about instance formatted like: "
                       "\"instance_path;transactions_days_number;admin_user;admin_password\"")
    (options, args) = parser.parse_args()
    if options.infos.startswith('#'):
        sys.exit(0)
    instdir, days, user, pwd = options.infos.split(';')
except ValueError:
    error("Problem in parameters")
    parser.print_help()
    sys.exit(1)

if __name__ == '__main__':
    #verbose("Begin of %s"%sys.argv[0])
    main()
    #verbose("End of %s"%sys.argv[0])



# -*- coding: utf-8 -*-
#
# File: CPUtilsTestCase.py
#
# Copyright (c) 2007 by CommunesPlone
# Generator: ArchGenXML Version 1.5.1-svn
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

__author__ = """Stephan GEULETTE <stephan.geulette@uvcw.be>"""
__docformat__ = 'plaintext'

#
# Base TestCase for contacts
#

import os, sys, code
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

##code-section module-header #fill in your manual code here
from Products.CMFCore.utils import getToolByName
from Products.CPUtils.config import *
from sets import Set

##/code-section module-header

from Testing import ZopeTestCase
from Products.PloneTestCase import PloneTestCase
ZopeTestCase.installProduct(PROJECTNAME)

PRODUCTS = list()
PRODUCTS.append(PROJECTNAME)

testcase = PloneTestCase.PloneTestCase

##code-section module-before-plone-site-setup #fill in your manual code here
##/code-section module-before-plone-site-setup

#Extension profile doesn't work because appinstall of ts appears to be not run
#PloneTestCase.setupPloneSite(products=PRODUCTS, extension_profiles=EXTENSION_PROFILES)
PloneTestCase.setupPloneSite(products=PRODUCTS)

class CPUtilsTestCase(testcase):
    """Base TestCase for contacts."""

    ##code-section class-header_CPUtilsTestCase #fill in your manual code here
    def afterSetup(self):
        """
        Manage users and permissions
        """
        self.wft = self.portal.portal_workflow
        uf = self.portal.acl_users
        uf.userFolderAddUser('member', 'member', ['Member', ], [])
        uf.userFolderAddUser('admin', 'admin', ['Manager', 'Member', ], [])
        uf.userFolderAddUser('anon', 'anon', ['Anonymous', ], [])
        #we have to be logged as a Manager to run methods from exportimport
        self.login('admin')
        #we run every import steps
#        self.portal.portal_setup.setImportContext("profile-Products.CPSkin2:default")
#        self.portal.portal_setup.runImportStep(step_id="createstructure-cpskin2")
#        self.portal.portal_setup.runImportStep(step_id="fillinleftslots-cpskin2")
        self.logout()

    def invokeFactoryC(self, type_name, id, path, **dict):
        path.invokeFactory(type_name=type_name, id=id, **dict)
        return getattr(path, id)

    def checkActionList(self, object, actions):
        """ Compare un set d'action de sorte que ['corriger', 'attendre'] soit egal a ['attendre', 'corriger'] """
        obj_actions = self.getActionList(object)
        self.assertEquals(Set(obj_actions), Set(actions))

    def getActionList(self, object):
        return [action_dict['name'] for action_dict in self.wft.getTransitionsFor(object)]


    ##/code-section class-header_CPUtilsTestCase

    # Commented out for now, it gets blasted at the moment anyway.
    # Place it in the protected section if you need it.
    #def afterSetup(self):
    #    """
    #    """
    #    pass

    def interact(self, locals=None):
        """Provides an interactive shell aka console inside your testcase.

        It looks exact like in a doctestcase and you can copy and paste
        code from the shell into your doctest. The locals in the testcase are
        available, becasue you are in the testcase.

        In your testcase or doctest you can invoke the shell at any point by
        calling::

            >>> self.interact( locals() )

        locals -- passed to InteractiveInterpreter.__init__()
        """
        savestdout = sys.stdout
        sys.stdout = sys.stderr
        sys.stderr.write('='*70)
        console = code.InteractiveConsole(locals)
        console.interact("""
ZopeTestCase Interactive Console
(c) BlueDynamics Alliance, Austria - 2005

Note: You have the same locals available as in your test-case.
""")
        sys.stdout.write('\nend of ZopeTestCase Interactive Console session\n')
        sys.stdout.write('='*70+'\n')
        sys.stdout = savestdout


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(CPUtilsTestCase))
    return suite

##code-section module-footer #fill in your manual code here
##/code-section module-footer

if __name__ == '__main__':
    framework()





import sys, os

from optparse import OptionParser
parser = OptionParser()
parser.add_option("-c", "--cache", action="store", type="string", dest="cache")
parser.add_option("-i", "--instance", action="store", type="string", dest="instance")
parser.add_option("-s", "--storage", action="store", type="string", dest="storage")
parser.add_option("-d", "--days", action="store", type="string", dest="days")
parser.add_option("-p", "--port", action="store", type="string", dest="port")

(options, args) = parser.parse_args()
cache = options.cache
instance = options.instance
storage = options.storage
days = options.days
port = options.port

                
sys.path[0:0] = [
    cache + '/eggs/plone.recipe.zope2zeoserver-1.4-py2.4.egg',
    cache + '/eggs/zc.recipe.egg-1.2.2-py2.4.egg',
    cache + '/eggs/setuptools-0.6c11-py2.4.egg',
    cache + '/eggs/zc.buildout-1.4.4-py2.4.egg',
    instance + '/parts/zope2/lib/python',
  ]

username = None
blob_dir = None
realm = None
unix = None
host = "127.0.0.1"
password = None



import plone.recipe.zope2zeoserver.pack

if __name__ == '__main__':
    plone.recipe.zope2zeoserver.pack.main(host, port, unix, days, username, password, realm, blob_dir, storage)
    



#!/usr/bin/python
#

import sys, os
import datetime
import shutil

buildout_inst_type = None #True for buildout, False for manual instance
zeo_type = False #True for zeo
was_running = True
ROTATE_CONF_FILE = 'logrotate.conf'
TMPDIR = '/tmp'

def verbose(*messages):
    print '>>', ' '.join(messages)
def error(*messages):
#    print >>sys.stderr, '!!', (' '.join(messages))
    print '!!', (' '.join(messages))

#------------------------------------------------------------------------------

def runCommand(cmd):
    """ run an os command and get back the stdout and stderr outputs """
    outfile = os.path.join(TMPDIR, '_cmd_zr.out')
    errfile = os.path.join(TMPDIR, '_cmd_zr.err')
    ret = os.system(cmd + ' >%s 2>%s'%(outfile, errfile))
    stdout = stderr = []
    try:
        if os.path.exists(outfile):
            ofile = open( outfile, 'r')
            stdout = ofile.readlines()
            ofile.close()
            os.remove(outfile)
        else:
            error("File %s does not exist" % outfile)
    except IOError:
        error("Cannot open %s file" % outfile)
    try:
        if os.path.exists(errfile):
            ifile = open( errfile, 'r')
            stderr = ifile.readlines()
            ifile.close()
            os.remove(errfile)
        else:
            error("File %s does not exist" % errfile)
    except IOError:
        error("Cannot open %s file" % errfile)
    return( stdout,stderr )

#------------------------------------------------------------------------------

def stop_instance(instance_section, path):
    """ Stop the instance """
    global was_running
    if buildout_inst_type:
        cmd = ''
        if zeo_type:
            cmd = path + '/bin/zeo stop;'
        cmd += path + '/bin/%s stop'%instance_section
    else:
        cmd = path + '/bin/zopectl stop'

    verbose("\tRunning command '%s'" % cmd)
    (cmd_out, cmd_err) = runCommand(cmd)

    if cmd_err:
        error("\t\tError when stopping instance : '%s'" % "".join(cmd_err))
    elif cmd_out:
        cmd_out = (''.join(cmd_out)).strip('\n ')
        if 'daemon process not running' in cmd_out or 'daemon manager not running' in cmd_out:
            was_running = False
            error('\t\tInstance not running')
        elif 'daemon process stopped' in cmd_out:
            verbose('\t\tWell stopped')
        else:
            verbose("\t\tOutput when stopping instance : '%s'" % "".join(cmd_out))
    else:
        error("\t\tNo output for command : '%s'" % cmd)

#------------------------------------------------------------------------------

def start_instance(instance_section, path):
    """ Start the instance """
    if not was_running:
        return
    if buildout_inst_type:
        cmd = ''
        if zeo_type:
            cmd = path + '/bin/zeo start;'
        cmd += path + '/bin/%s start'%instance_section
    else:
        cmd = path + '/bin/zopectl start'

    verbose("\tRunning command '%s'" % cmd)
    (cmd_out, cmd_err) = runCommand(cmd)

    if cmd_err:
        error("\t\tError when starting instance : '%s'" % "".join(cmd_err))
    elif cmd_out:
        cmd_out = (''.join(cmd_out)).strip('\n ')
        if 'daemon process started' in cmd_out:
            verbose('\t\tWell started : %s'%cmd_out)
        else:
            verbose("\t\tOutput when starting instance : '%s'" % "".join(cmd_out))
    else:
        error("\t\tNo output for command : '%s'" % cmd)

#------------------------------------------------------------------------------

def rotate_logs(path):
    """
        Rotate logs of the instance
    """
    verbose("\tLog rotation on %s"%path)
    rotate_conf = os.path.join(path, ROTATE_CONF_FILE)
    if not os.path.exists(rotate_conf):
        error("\t\tRotate conf file doesn't exist '%s'"%rotate_conf)
        return
#    cmd = '/usr/sbin/logrotate -d ' #debug
    cmd = '/usr/sbin/logrotate '
    cmd += rotate_conf

    verbose("\tRunning command '%s'" % cmd)
    (cmd_out, cmd_err) = runCommand(cmd)

    if cmd_err:
        error("\t\tError when running logrotate : '%s'" % "".join(cmd_err))
    elif cmd_out:
        verbose("\t\tOutput when running logrotate : '%s'" % "".join(cmd_out))
    else:
        pass
        #error("\t\tNo output for command : '%s'" % cmd)

#------------------------------------------------------------------------------

def main():
    global buildout_inst_type, zeo_type
    instance_section = 'instance'

    tmp = instdir
    if tmp.endswith('/'):
        tmp = tmp[0:-1]
    verbose("Working on instance %s" % tmp)

    if os.path.exists(os.path.join(tmp,'parts')):
        buildout_inst_type = True
        verbose("\tInstance is a buildout !")
    elif os.path.exists(os.path.join(tmp,'etc')):
        buildout_inst_type = False
        verbose("\tInstance is a manual installation !")
    elif not os.path.exists(tmp) or True:
        error("! Invalid instance path '%s' or instance type not detected"%tmp)
        sys.exit(1)

    if buildout_inst_type:
        if os.path.exists(os.path.join(tmp, 'bin', 'zeo')):
            zeo_type = True
            verbose("\tInstance is a zeo !")
        if os.path.exists(os.path.join(tmp, 'bin', 'instance1')):
            instance_section = 'instance1'

    if not subcommand:
        # 1. Stop the instance if running
        stop_instance(instance_section, tmp)

        # 2. Rotate the logs of the instance
        rotate_logs(tmp)

        # 3. Restart the instance if it was running
        start_instance(instance_section, tmp)
    else:
        if buildout_inst_type:
            cmd = ''
            if zeo_type:
                cmd = tmp + '/bin/zeo %s;'%subcommand
            cmd += tmp + '/bin/%s %s'%(instance_section, subcommand)
        else:
            cmd = tmp + '/bin/zopectl %s'%subcommand

        verbose("\tRunning command '%s'" % cmd)
        (cmd_out, cmd_err) = runCommand(cmd)
        verbose("\toutput: '%s'" % "".join(cmd_out))

#------------------------------------------------------------------------------

try:
    from optparse import OptionParser
    parser = OptionParser()
    parser.add_option("-i", "--infos", dest="infos",
                  default=None,
                  help="infos about instance formatted like: "
                       "\"instance_path;transactions_days_number;admin_user;admin_password\"")
    parser.add_option("-c", "--command", dest="subcommand",
                  help="subcommand name like: start, stop, restart, status",
                  default='')
    (options, args) = parser.parse_args()
    if options.infos.startswith('#'):
        sys.exit(0)
    instdir, days, user, pwd = options.infos.split(';')
    subcommand = options.subcommand
except ValueError:
    error("Problem in parameters")
    parser.print_help()
    sys.exit(1)

if __name__ == '__main__':
    main()
